library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library WORK;
use WORK.config.all;


entity ConfProto is
	PORT(
		-- GENERIC IOs
		CLK		: in  std_logic;
		CE		: in  std_logic;
		RESET	: in  std_logic;

		-- INTERNAL SPI IOs
		DAT_TX		: out std_logic_vector (31 downto 0);
		DAT_RX		: in  std_logic_vector (31 downto 0);
		TRANS_DONE: in  std_logic;
		DAT_VALID	: out std_logic;

		-- CONFIG IOs
		CONF_MODE	: out std_logic;
		WE				: out std_logic;
		DAT_W			: out std_logic_vector (31 downto 0);
		ADR				: out integer range 0 to (RAM_SIZE/4)-1;
		DAT_SIZE	: out integer range 0 to (RAM_SIZE/4)-1
	);
end entity ConfProto;


architecture Behavioral of ConfProto is

	type STATE_t is (IDLE, DATA, FIN, ERROR);
	signal STATE : STATE_t := IDLE;

	signal COUNTER : integer range 0 to (RAM_SIZE/4)+1;
	signal ADR_b : integer range 0 to (RAM_SIZE/4)-1;
	signal LENGTH : std_logic_vector (15 downto 0) := (others => '0');

begin

	process (CLK) is
	begin

		if RESET = '1' then

			STATE <= IDLE;
			COUNTER <= 0;
			LENGTH <= (others => '0');
			ADR_b <= 0;
			ADR <= 0;
			WE <= '0';
			DAT_TX <= (others => '0');
			CONF_MODE <= '1';

		elsif rising_edge(CLK) and CE = '1' then
			if TRANS_DONE = '1' then

			DAT_VALID <= '1';

			case STATE is
				when IDLE =>
					if DAT_RX(31 downto 16) = CONF_PROTO_INIT then
						STATE <= DATA;
						DAT_TX <= CONF_PROTO_ACK;
						CONF_MODE <= '1';
					else
						STATE <= ERROR;
						DAT_TX <= CONF_PROTO_ERR;
					end if;
					LENGTH <= DAT_RX(15 downto 0);
					COUNTER <= 2;
					ADR_b <= 0;
					WE <= '0';

				when DATA =>
					if COUNTER > to_integer(unsigned(LENGTH)) then
						STATE <= FIN;
					end if;
					DAT_TX <= CONF_PROTO_ACK;
					COUNTER <= COUNTER + 1;
					WE <= '1';
					DAT_W <= DAT_RX;
					ADR_b <= ADR_b + 1;

				when FIN =>
					if DAT_RX = CONF_PROTO_FIN then
						STATE <= IDLE;
						DAT_TX <= CONF_PROTO_ACK;
						CONF_MODE <= '0';
						DAT_SIZE <= to_integer(unsigned(LENGTH) - 1);
					else
						STATE <= ERROR;
						DAT_TX <= CONF_PROTO_ERR;
					end if;
					WE <= '0';

				when ERROR =>
					if DAT_RX = CONF_PROTO_RECOVER then
						STATE <= IDLE;
						DAT_TX <= CONF_PROTO_ACK;
					end if;

			end case;

			else -- TRANS_DONE = '0'
				DAT_VALID <= '0';
				WE <= '0';
			end if;

			ADR <= ADR_b;

		end if;

	end process;


end architecture;
