library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library WORK;
use WORK.config.all;


entity ConfProto_TB is
end entity ConfProto_TB;


architecture Simul of ConfProto_TB is

	component ConfProto
		PORT(
			-- GENERIC IOs
			CLK		: in  std_logic;
			CE		: in  std_logic;
			RESET	: in  std_logic;

			-- INTERNAL SPI IOs
			DAT_TX		: out std_logic_vector (31 downto 0);
			DAT_RX		: in  std_logic_vector (31 downto 0);
			TRANS_DONE: in  std_logic;
			DAT_VALID	: out std_logic;

			-- CONFIG IOs
			CONF_MODE	: out std_logic;
			WE				: out std_logic;
			DAT_W			: out std_logic_vector (31 downto 0);
			ADR				: out integer range 0 to (RAM_SIZE/4)-1;
			DAT_SIZE	: out integer range 0 to (RAM_SIZE/4)-1
		);
	end component;

	-- INPUTS
	signal CLK				: std_logic := '0';
	signal CE					: std_logic := '1';
	signal RESET			: std_logic := '1';
	signal DAT_RX			: std_logic_vector (31 downto 0) := (others => '0');
	signal TRANS_DONE	: std_logic := '0';


	-- OUTPUTS
	signal DAT_TX			: std_logic_vector (31 downto 0);
	signal DAT_VALID	: std_logic;
	signal CONF_MODE	: std_logic;
	signal WE					: std_logic;
	signal DAT_W			: std_logic_vector (31 downto 0);
	signal ADR				: integer range 0 to (RAM_SIZE/4)-1;
	signal DAT_SIZE		: integer range 0 to (RAM_SIZE/4)-1;

	-- CONSTANTS
	constant CLK_PERIOD : time := 10 ns;

begin

	UUT : ConfProto
		port map(
			CLK => CLK,
			CE => CE,
			RESET => RESET,

			DAT_TX => DAT_TX,
			DAT_RX => DAT_RX,
			TRANS_DONE => TRANS_DONE,
			DAT_VALID => DAT_VALID,

			CONF_MODE => CONF_MODE,
			WE => WE,
			DAT_W => DAT_W,
			ADR => ADR,
			DAT_SIZE => DAT_SIZE
		);


	clk_stim : process
	begin
		CLK <= '0';
		wait for CLK_PERIOD/2;
		CLK <= '1';
		wait for CLK_PERIOD/2;
	end process;


	process
	begin
		wait for 10 ns;
		RESET <= '0';

		wait for 20 ns;

		DAT_RX <= CONF_PROTO_INIT & x"0002";
		TRANS_DONE <= '1';
		wait for CLK_PERIOD;
		TRANS_DONE <= '0';
		wait for 100 ns;

		DAT_RX <= x"ABCD1234";
		TRANS_DONE <= '1';
		wait for CLK_PERIOD;
		TRANS_DONE <= '0';
		wait for 100 ns;

		DAT_RX <= x"43210987";
		TRANS_DONE <= '1';
		wait for CLK_PERIOD;
		TRANS_DONE <= '0';
		wait for 100 ns;

		DAT_RX <= x"44440987";
		TRANS_DONE <= '1';
		wait for CLK_PERIOD;
		TRANS_DONE <= '0';
		wait for 100 ns;

		DAT_RX <= CONF_PROTO_FIN;
		TRANS_DONE <= '1';
		wait for CLK_PERIOD;
		TRANS_DONE <= '0';
		wait for 100 ns;

		DAT_RX <= CONF_PROTO_RECOVER;
		TRANS_DONE <= '1';
		wait for CLK_PERIOD;
		TRANS_DONE <= '0';
		wait for 100 ns;

		wait;

	end process;

end architecture;
