library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library WORK;
use WORK.config.all;

entity PulseGen is
	PORT(
		-- GENERIC IOs
		CLK		: in std_logic;
		CE		: in std_logic;
		RESET	: in std_logic;
		O			: out std_logic;

		-- CONFIG IOs
		CONF_MODE	: in std_logic;
		WE				: in std_logic;
		DAT_W			: in std_logic_vector (31 downto 0);
		ADR				: in integer range 0 to (RAM_SIZE/4)-1;
		DAT_SIZE	: in integer range 0 to (RAM_SIZE/4)-1
	);
end entity PulseGen;


architecture Behavioral of PulseGen is

	signal RLC_COUNTER	: integer range 0 to (2**30) := 1;
	signal NRLC_COUNTER	: integer range 0 to 30 := 0;
	signal CNT_PulseGen	: integer range 0 to (2**30)-1 := 0;
	signal CUR_DAT	: std_logic_vector (31 downto 0) := (OTHERS => '0');
	signal NEXT_DAT	: std_logic_vector (31 downto 0) := (OTHERS => '0');
	signal REQ_DAT	: std_logic := '0';

	type RAM_t is array (natural range <>) of std_logic_vector (31 downto 0);
	signal RAM			: RAM_t ((RAM_SIZE/4)-1 downto 0) := (others => (others => '0'));
	signal READ_ADR	: integer range 0 to (RAM_SIZE/4)-1;

begin


	process (CLK) is
	begin

		if RESET = '1' then

				RLC_COUNTER <= 1;
				NRLC_COUNTER <= 0;
				CNT_PulseGen <= 0;
				CUR_DAT <= (others => '0');
				-- Dirty Workaround: Xilinx tools need this register to have a sync. reset, while Altera tools need it to have an async reset
				-- We don't need an initial Value since we'll go into config mode on reset anyways.
				-- NEXT_DAT <= (others => '0');
				REQ_DAT <= '0';
				READ_ADR <= 0;
				O <= '0';

		elsif rising_edge(CLK) and CE = '1' then

			if CONF_MODE = '0' then

				if CUR_DAT(31) = '0' then
				-- NON RLC Word
					if NRLC_COUNTER = 30 then
						NRLC_COUNTER <= 0;
						CUR_DAT <= NEXT_DAT;
						CNT_PulseGen <= to_integer(unsigned(NEXT_DAT(29 downto 0)));
						REQ_DAT <= '1';
					else
						NRLC_COUNTER <= NRLC_COUNTER + 1;
					end if;
					O <= CUR_DAT(NRLC_COUNTER);	-- SREG might be faster/smaller than MUX here

				else -- CUR_DAT(31) = '1'
				-- RLC WORD
					if RLC_COUNTER = CNT_PulseGen then
						RLC_COUNTER <= 1;
						CUR_DAT <= NEXT_DAT;
						CNT_PulseGen <= to_integer(unsigned(NEXT_DAT(29 downto 0)));
						REQ_DAT <= '1';
					else
						RLC_COUNTER <= RLC_COUNTER + 1;
					end if;
					O <= CUR_DAT(30);
				end if;


				if REQ_DAT = '1' then
					NEXT_DAT <= RAM(READ_ADR);
					if READ_ADR = DAT_SIZE then
						READ_ADR <= 0;
					else
						READ_ADR <= READ_ADR + 1;
					end if;
					REQ_DAT <= '0';
				end if;

			else -- CONF_MODE = '1'

				RLC_COUNTER <= 1;
				NRLC_COUNTER <= 0;
				CNT_PulseGen <= 0;
				CUR_DAT <= (others => '0');
				NEXT_DAT <= (others => '0');
				REQ_DAT <= '0';
				READ_ADR <= 0;
				O <= '0';

				if WE = '1' then
					RAM(ADR) <= DAT_W;
				end if;


			end if;
		end if;
	end process;


end architecture;
