library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library WORK;
use WORK.config.all;

entity PulseGen_TB is
end entity PulseGen_TB;


architecture Simul of PulseGen_TB is

	component PulseGen
		PORT(
			-- GENERIC IOs
			CLK		: in std_logic;
			CE		: in std_logic;
			RESET	: in std_logic;
			O			: out std_logic;

			-- CONFIG IOs
			CONF_MODE	: in std_logic;
			WE				: in std_logic;
			DAT_W			: in std_logic_vector (31 downto 0);
			ADR				: in integer range 0 to (RAM_SIZE/4)-1;
			DAT_SIZE	: in integer range 0 to (RAM_SIZE/4)-1
		);
	end component;

	-- INPUTS
	signal CLK				: std_logic := '0';
	signal CE					: std_logic := '1';
	signal RESET			: std_logic := '1';
	signal CONF_MODE	: std_logic := '0';
	signal WE					: std_logic := '0';
	signal DAT_W			: std_logic_vector (31 downto 0) := (others => '0');
	signal ADR				: integer range 0 to (RAM_SIZE/4)-1 := 0;
	signal DAT_SIZE		: integer range 0 to (RAM_SIZE/4)-1 := 0;

	-- OUTPUTS
	signal O	: std_logic;

	-- CONSTANTS
	constant CLK_PERIOD : time := 10 ns;

begin

	UUT : PulseGen
		port map(
			CLK => CLK,
			CE => CE,
			RESET => RESET,
			O => O,

			CONF_MODE => CONF_MODE,
			WE => WE,
			DAT_W => DAT_W,
			ADR => ADR,
			DAT_SIZE => DAT_SIZE
		);

	clk_stim : process
	begin
		CLK <= '0';
		wait for CLK_PERIOD/2;
		CLK <= '1';
		wait for CLK_PERIOD/2;
	end process;


	process
	begin
		wait for 10 ns;
		RESET <= '0';
		CONF_MODE <= '1';
		DAT_SIZE <= 1;

		wait until rising_edge(clk);
		DAT_W <= x"C0000002";
		ADR <= 0;
		WE <= '1';

		wait until rising_edge(clk);
		DAT_W <= x"80000002";
		ADR <= 1;
		WE <= '1';

		wait until rising_edge(clk);
		DAT_W <= x"7FFFFFFF";
		ADR <= 2;
		WE <= '1';

		wait until rising_edge(clk);
		WE <= '0';

		wait for 100 ns;
		CONF_MODE <= '0';

		wait;

	end process;

end architecture;
