library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library WORK;
use WORK.config.all;


entity SpiCtrl is
	PORT(
		-- GENERIC IOs
		CLK		: in  std_logic;
		CE		: in  std_logic;
		RESET	: in  std_logic;

		-- SPI IOs
		SPI_CLK		: in  std_logic;
		SPI_MISO	: out std_logic;
		SPI_MOSI	: in  std_logic;
		SPI_SS		: in  std_logic;

		-- INTERNAL IOs
		DAT_TX		: in  std_logic_vector (31 downto 0);
		DAT_RX		: out std_logic_vector (31 downto 0);
		TRANS_DONE: out std_logic;
		DAT_VALID	: in  std_logic
	);
end entity SpiCtrl;


architecture Behavioral of SpiCtrl is

	type STATE_t is (IDLE, DATA);
	signal STATE : STATE_t := DATA;

	signal COUNTER : integer range 0 to 31 := 0;
	signal SPI_MISO_b : std_logic := '0';
	signal SPI_CLK_old : std_logic := '0';
	signal SREG : std_logic_vector (31 downto 0) := x"0AC0AC0A";

begin

	process (CLK) is
	begin

		if RESET = '1' then

			STATE <= DATA;
			COUNTER <= 0;
			SPI_MISO_b <= '0';
			SPI_CLK_old <= '0';
			SREG <= x"0AC0AC0A";

		elsif rising_edge(CLK) and CE = '1' then

			case STATE is

				when IDLE =>
					if DAT_VALID = '1' then
						SREG <= DAT_TX;
						SPI_MISO_b <= DAT_TX(31);
						STATE <= DATA;
					end if;
					COUNTER <= 0;
					SPI_CLK_old <= '0';
					TRANS_DONE <= '0';

				when DATA =>
					-- READ ON THE RISING EDGE
					if SPI_CLK_old = '0' and SPI_CLK = '1' and SPI_SS = '1' then
						SREG(31 downto 0) <= SREG(30 downto 0) & SPI_MOSI;
					end if;

					-- WRITE ON THE FALLING EDGE
					if SPI_CLK_old = '1' and SPI_CLK = '0' and SPI_SS = '1' then
						if COUNTER = 31 then
							STATE <= IDLE;
							DAT_RX <= SREG;
							TRANS_DONE <= '1';
							COUNTER <= 0;
						else
							COUNTER <= COUNTER + 1;
							SPI_MISO_b <= SREG(31);
						end if;
					end if;

					if SPI_SS = '0' then
						-- GO TO AN ERROR STATE, IF COUNTER IS NOT DIVISIBLE BY 8 MAYBE?
					end if;

					SPI_CLK_old <= SPI_CLK;

			end case;

			if SPI_SS = '1' then
				SPI_MISO <= SPI_MISO_b;
			else
				SPI_MISO <= 'Z';
			end if;

		end if;

	end process;

end architecture;
