library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library WORK;
use WORK.config.all;


entity SpiCtrl_TB is
end entity SpiCtrl_TB;


architecture Simul of SpiCtrl_TB is

	component SpiCtrl
		PORT(
			-- GENERIC IOs
			CLK		: in  std_logic;
			CE		: in  std_logic;
			RESET	: in  std_logic;

			-- SPI IOs
			SPI_CLK		: in  std_logic;
			SPI_MISO	: out std_logic;
			SPI_MOSI	: in  std_logic;
			SPI_SS		: in  std_logic;

			-- INTERNAL IOs
			DAT_TX		: in  std_logic_vector (31 downto 0);
			DAT_RX		: out std_logic_vector (31 downto 0);
			TRANS_DONE: out std_logic;
			DAT_VALID	: in  std_logic
		);
	end component;

	-- INPUTS
	signal CLK				: std_logic := '0';
	signal CE					: std_logic := '1';
	signal RESET			: std_logic := '1';

	signal SPI_CLK		: std_logic := '0';
	signal SPI_MOSI		: std_logic := '0';
	signal SPI_SS			: std_logic := '0';

	signal DAT_TX			: std_logic_vector (31 downto 0) := (others => '0');
	signal DAT_VALID	: std_logic := '0';

	-- OUTPUTS
	signal SPI_MISO		: std_logic;
	signal DAT_RX			: std_logic_vector (31 downto 0);
	signal TRANS_DONE	: std_logic;

	-- CONSTANTS
	constant CLK_PERIOD : time := 10 ns;

begin

	UUT : SpiCtrl
		port map(
			CLK => CLK,
			CE => CE,
			RESET => RESET,

			SPI_CLK => SPI_CLK,
			SPI_MISO => SPI_MISO,
			SPI_MOSI => SPI_MOSI,
			SPI_SS => SPI_SS,

			DAT_TX => DAT_TX,
			DAT_RX => DAT_RX,
			TRANS_DONE => TRANS_DONE,
			DAT_VALID => DAT_VALID
		);

	clk_stim : process
	begin
		CLK <= '0';
		wait for CLK_PERIOD/2;
		CLK <= '1';
		wait for CLK_PERIOD/2;
	end process;


	process
	begin
		wait for 10 ns;
		RESET <= '0';

		wait for 10 ns;
		SPI_SS <= '1';
		SPI_MOSI <= '1';
		DAT_VALID <= '1';
		wait for CLK_PERIOD;
		DAT_VALID <= '0';

		for i in 0 to 31 loop
			wait for CLK_PERIOD*3;
			SPI_CLK <= '1';
			wait for CLK_PERIOD*3;
			SPI_CLK <= '0';
		end loop;

		wait;

	end process;

end architecture;
