library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library WORK;
use WORK.config.all;


entity Top_TB is
end entity Top_TB;


architecture Simul of Top_TB is

	component Top
		PORT(
			-- GENERIC IOs
			CLK		: in  std_logic;
			RESET	: in  std_logic;

			-- SPI IOs
			SPI_CLK		: in  std_logic;
			SPI_MISO	: out std_logic;
			SPI_MOSI	: in  std_logic;
			SPI_SS		: in  std_logic;

			O			: out std_logic
		);
	end component;

	-- INPUTS
	signal CLK : std_logic := '0';
	signal RESET : std_logic := '1';
	signal SPI_CLK : std_logic := '0';
	signal SPI_MOSI : std_logic := '0';
	signal SPI_SS : std_logic := '0';

	-- OUTPUTS
	signal SPI_MISO : std_logic;
	signal O : std_logic;

	-- CONSTANTS
	constant CLK_PERIOD : time := 5 ns;

begin

	UUT : Top
		port map(
			CLK => CLK,
			RESET => RESET,

			SPI_CLK => SPI_CLK,
			SPI_MISO => SPI_MISO,
			SPI_MOSI => SPI_MOSI,
			SPI_SS => SPI_SS,

			O => O
		);

	clk_stim : process
	begin
		CLK <= '0';
		wait for CLK_PERIOD/2;
		CLK <= '1';
		wait for CLK_PERIOD/2;
	end process;


	process

	procedure SPI_WRITE (
		TX : in  std_logic_vector (31 downto 0);
		RX : out std_logic_vector (31 downto 0)
		) is
		variable RX_b : std_logic_vector (31 downto 0);
	begin
		wait for CLK_PERIOD*20;
		SPI_SS <= '1';
		wait for CLK_PERIOD*20;
		for i in 0 to 31 loop
			SPI_MOSI <= TX(31-i);
			RX_b(31-i) := SPI_MISO;
			SPI_CLK <= '1';
			wait for CLK_PERIOD*20;
			SPI_CLK <= '0';
			wait for CLK_PERIOD*20;
		end loop;
		wait for CLK_PERIOD*20;
		SPI_SS <= '0';
		RX := RX_b;
	end procedure;

	variable SPI_RET : std_logic_vector (31 downto 0);

	begin
		wait for 10 ns;
		RESET <= '0';

		wait for 20 ns;
		SPI_WRITE(CONF_PROTO_INIT & x"0005", SPI_RET);
		SPI_WRITE(x"0AAFFAA0", SPI_RET);
		SPI_WRITE(x"00123456", SPI_RET);
		SPI_WRITE(x"C0000040", SPI_RET);
		SPI_WRITE(x"80000500", SPI_RET);
		SPI_WRITE(x"0A0A0A0F", SPI_RET);
		SPI_WRITE(CONF_PROTO_FIN, SPI_RET);
		wait;
	end process;

end architecture;
